// unnamed package

import java.util.Random;
import java.util.Vector;
import javax.microedition.lcdui.*;
import javax.microedition.lcdui.game.*;
import javax.microedition.media.*;
import java.io.*;


class SheepdogCanvas
    extends GameCanvas
    implements Runnable
{
    // shared direction constants
    static final int NONE = -1;
    static final int UP = 0;
    static final int LEFT = 1;
    static final int DOWN = 2;
    static final int RIGHT = 3;

    private static final int MILLIS_PER_TICK = 50;
    private static final int NUM_SHEEP = 20;
    private static final int dist = 15;

    private final SheepdogMIDlet midlet;
    private final Field field;
    private final Sheepdog sheepdog;
    private final Vector sheep = new Vector();
    private final LayerManager layerManager;

    private final Graphics graphics;
    private long gameDuration;
    private long startTime;
    private long oldTime=0;
    private long frames=0;
    private long fps=0;
    private volatile Thread animationThread = null;

    private boolean drawMap = false;



    SheepdogCanvas(SheepdogMIDlet midlet)
    {
        super(true);   // suppress key events for game keys
        this.midlet = midlet;
        setFullScreenMode(true);
        graphics = getGraphics();

        layerManager = new LayerManager();
        field = new Field(this);
        sheepdog = new Sheepdog(this);
        layerManager.append(sheepdog);
        for (int i = 0; i < NUM_SHEEP; ++i)
        {
            Sheep newSheep = new Sheep(this);
            layerManager.append(newSheep);
            sheep.addElement(newSheep);
        }

        layerManager.append(field);   // last layer, behind sprites

        init();
    }


    public void keyPressed(int keyCode)
    {
        // The constructor suppresses key events for game keys, so we'll
        // only get key events for non-game keys. The number keys, * & #
        // have positive keyCodes, so negative keyCodes mean non-game
        // special keys like soft-keys. We'll use key-presses on special
        // keys to take us to the menu.
        if (keyCode < 0)
        {
            stop();
            midlet.sheepdogCanvasMenu();
        }
        if( keyCode == 1 ) {drawMap=!drawMap;}
    }

    void init()
    {
        sheepdog.setPosition(field.getSheepdogStartX(),
                             field.getSheepdogStartY());

        for (int i = 0; i < sheep.size(); ++i)
        {
            Sheep sh = (Sheep)(sheep.elementAt(i));
            repositionSheep(sh,i);
        }
    }


    public synchronized void start()
    {
        animationThread = new Thread(this);
        animationThread.start();

        startTime = System.currentTimeMillis() - gameDuration;
    }


    public synchronized void stop()
    {
        gameDuration = System.currentTimeMillis() - startTime;
        animationThread = null;
    }


    public void run()
    {
        Thread currentThread = Thread.currentThread();

        try
        {
            // This ends when animationThread is set to null, or when
            // it is subsequently set to a new thread; either way, the
            // current thread should terminate
            while (currentThread == animationThread)
            {
                long startTime = System.currentTimeMillis();
                // Don't advance game or draw if canvas is covered by
                // a system screen.
                if (isShown())
                {
                    tick();
                    draw();
                    flushGraphics();
                }
                long timeTaken = System.currentTimeMillis() - startTime;
                if (timeTaken < MILLIS_PER_TICK)
                {
                    synchronized (this)
                    {
                        wait(MILLIS_PER_TICK - timeTaken);
                    }
                }
                else
                {
                    currentThread.yield();
                }
            }
        }
        catch (InterruptedException e)
        {
            // won't be thrown
        }
    }


    private void tick()
    {
        // If player presses two or more direction buttons, we ignore them
        // all. But pressing fire is independent. The code below also ignores
        // direction buttons if GAME_A..GAME_D are pressed.
        int keyStates = getKeyStates();
        boolean bark = false;
//        (keyStates & FIRE_PRESSED) != 0;
//        keyStates &= ~FIRE_PRESSED;
        int direction = (keyStates == UP_PRESSED) ? UP :
                        (keyStates == LEFT_PRESSED) ? LEFT:
                        (keyStates == DOWN_PRESSED) ? DOWN :
                        (keyStates == RIGHT_PRESSED) ? RIGHT : NONE;
        if( ((keyStates & UP_PRESSED ) != 0 ) ||
            (((keyStates & LEFT_PRESSED ) != 0) && ((keyStates & RIGHT_PRESSED ) != 0)))
          sheepdog.speedUp();
        if( (keyStates & DOWN_PRESSED ) != 0 )
          sheepdog.speedDown();
        if( (keyStates & LEFT_PRESSED ) != 0 )
          sheepdog.left();
        if( (keyStates & RIGHT_PRESSED ) != 0 )
          sheepdog.right();
        if( (keyStates & FIRE_PRESSED) != 0 )
          drawMap=true;
        else
          drawMap=false;

        sheepdog.tick(direction, bark);

        for (int i = 0; i < sheep.size(); ++i)
        {
            Sheep sh = (Sheep)(sheep.elementAt(i));
            int dx = sheepdog.getX()-sh.getX();
            int dy = sheepdog.getY()-sh.getY();
            if( dx>dist*Field.TILE_WIDTH || dy>dist*Field.TILE_HEIGHT || dx<-dist*Field.TILE_WIDTH || dy<-dist*Field.TILE_HEIGHT )
              repositionSheep(sh,i);
            sh.tick();
        }

        field.tick();
    }


    public Field getField()
    {
        return field;
    }


    public SheepdogMIDlet getMidlet()
    {
        return midlet;
    }

    Sheepdog getSheepdog()
    {
        return sheepdog;
    }


    Vector getSheep()
    {
        return sheep;
    }


    boolean overlapsSheepdog(Sprite sprite)
    {
        return sprite.collidesWith(sheepdog, false); // false -> not pixelLevel
    }


    boolean overlapsSheep(Sprite sprite)
    {
        return overlapsSheep(sprite, sheep.size());
    }


    // whether the sprite overlaps the first 'count' sheep
    boolean overlapsSheep(Sprite sprite, int count)
    {
        for (int i = 0; i < count; ++i)
        {
            Sheep sh = (Sheep)(sheep.elementAt(i));
            if (sprite.collidesWith(sh, false))  // false -> not pixelLevel
            {
                return true;
            }
        }
        return false;
    }


    boolean overlapsOtherSheep(Sprite sprite)
    {
        for (int i = 0; i < sheep.size(); ++i)
        {
            Object obj = sheep.elementAt(i);
            if (obj != sprite)
            {
                Sheep sh = (Sheep)obj;
                if (sprite.collidesWith(sh, false))  // false -> not pixelLevel
                {
                    return true;
                }
            }
        }
        return false;
    }


    void vibrate(int millis)
    {
        midlet.vibrate(millis);
    }


    // draw game
    private void draw()
    {
        int width = getWidth();
        int height = getHeight();

        // clear screen to grey
        graphics.setColor(0x00888888);
        graphics.fillRect(0, 0, width, height);

        if( drawMap ) {
          graphics.setClip(0, 0, width, height);
          for( int i=0; i<field.WIDTH_IN_TILES; i++)
            for( int j=0; j<field.HEIGHT_IN_TILES; j++)
              if( field.isIn(field.getElemColRow(i,j),field.ROAD_TILES) ) {
                graphics.setColor(200,200,200);
                graphics.drawLine(i, j, i, j);
              }
              else
              if( field.isIn(field.getElemColRow(i,j),field.WATER_TILES) ) {
                graphics.setColor(0,100,255);
                graphics.drawLine(i, j, i, j);
              }
              else
              if( field.isIn(field.getElemColRow(i,j),field.HOUSE_TILES) ) {
                if( field.getElemColRow(i,j) == 32 ) {
                  graphics.setColor(255, 255, 255);
                  graphics.drawLine(i-1, j, i+1, j);
                  graphics.drawLine(i, j-1, i, j+1);
                }
                else
                  graphics.setColor(255,0,0);
                graphics.drawLine(i, j, i, j);
              }
              else {
                graphics.setColor(0,255,50);
                graphics.drawLine(i, j, i, j);
              }
          graphics.setColor(255,255,255);
          for (int i = 0; i < sheep.size(); ++i)
          {
              Sheep sh = (Sheep)(sheep.elementAt(i));
              graphics.drawLine(sh.getX()/field.TILE_WIDTH-1, sh.getY()/field.TILE_HEIGHT,sh.getX()/field.TILE_WIDTH+1, sh.getY()/field.TILE_HEIGHT);
              graphics.drawLine(sh.getX()/field.TILE_WIDTH, sh.getY()/field.TILE_HEIGHT-1,sh.getX()/field.TILE_WIDTH, sh.getY()/field.TILE_HEIGHT+1);
          }
          graphics.setColor(255,255,255);
          graphics.drawLine(sheepdog.getX()/field.TILE_WIDTH-3, sheepdog.getY()/field.TILE_HEIGHT,sheepdog.getX()/field.TILE_WIDTH+3, sheepdog.getY()/field.TILE_HEIGHT);
          graphics.drawLine(sheepdog.getX()/field.TILE_WIDTH, sheepdog.getY()/field.TILE_HEIGHT-3,sheepdog.getX()/field.TILE_WIDTH, sheepdog.getY()/field.TILE_HEIGHT+3);
          return;
        }

        // clip and translate to centre
        int dx = origin(sheepdog.getX() + sheepdog.getWidth() / 2,
                        field.getWidth(),
                        width);
        int dy = origin(sheepdog.getY() + sheepdog.getHeight() / 2,
                        field.getHeight(),
                        height);
        graphics.setClip(dx, dy, field.getWidth(), field.getHeight());
        graphics.translate(dx, dy);

        // draw background and sprites
        layerManager.paint(graphics, 0, 0);

        // undo clip & translate
        graphics.translate(-dx, -dy);
        graphics.setClip(0, 0, width, height);

        // display time & score
        long time = (System.currentTimeMillis() - startTime) / 1000;
        if( oldTime != time ) {
          oldTime=time;
          fps=frames;
          frames=0;
        } else frames++;

        int score = sheepdog.getSpeed();
//            field.getElem(sheepdog.getX()/field.TILE_WIDTH,sheepdog.getY()/field.TILE_HEIGHT);
        graphics.setColor(0x00FFFFFF); // white
        graphics.drawString("Speed: "+Integer.toString(score)+"Kph",
                            1,
                            1,
                            Graphics.TOP | Graphics.LEFT);
        graphics.drawString("FPS: "+Long.toString(fps),
                            width - 2,
                            1,
                            Graphics.TOP | Graphics.RIGHT);
/*
        if (score == sheep.size())
        {
            midlet.sheepdogCanvasGameOver(time);
        }
*/    }


    // If the screen is bigger than the field, we center the field
    // in the screen. Otherwise we center the screen on the focus, except
    // that we don't scroll beyond the edges of the field.
    private int origin(int focus, int fieldLength, int screenLength)
    {
        int origin;
        if (screenLength >= fieldLength)
        {
            origin = (screenLength - fieldLength) / 2;
        }
        else if (focus <= screenLength / 2)
        {
            origin = 0;
        }
        else if (focus >= (fieldLength - screenLength / 2))
        {
            origin = screenLength - fieldLength;
        }
        else
        {
            origin = screenLength / 2 - focus;
        }
        return origin;
    }


    int numSheepInFold()
    {
        int count = 0;
        for (int i = 0; i < sheep.size(); ++i)
        {
            Sheep sh = (Sheep)(sheep.elementAt(i));
            if (field.inFold(sh))
            {
                count++;
            }
        }
        return count;
    }

    public void repositionSheep(Sheep sh, int i) {
      boolean mehet = false;
      int rad = 0;
      // find a valid position for the sheep
      do
      {

          int x = midlet.random((dist-5+rad)*2);
          int y = midlet.random((dist-5+rad)*2);
          x -= (dist+rad);y -= (dist+rad);
          if(x>=0) x+=5; else x-=5;
          if(y>=0) y+=5; else y-=5;
          x += sheepdog.getX()/field.TILE_WIDTH;
          y += sheepdog.getY()/field.TILE_HEIGHT;
          if( x<0 || y<0 || x>=Field.WIDTH_IN_TILES || y>=Field.HEIGHT_IN_TILES ) {
            x=0;y=0;
          }
//                x = 3; y = 5;
          sh.setSpPosition(field.TILE_WIDTH * x +1,
                           field.TILE_HEIGHT * y +1);
          int elem = field.getElemColRow(x,y);
          if( Field.isIn(elem,field.UP_TILES) ) {
            sh.setDirection(0);
            mehet=true;
          } else if( Field.isIn(elem,field.DOWN_TILES) ) {
            sh.setDirection(64);
            mehet=true;
          } else if( Field.isIn(elem,field.LEFT_TILES) ) {
            sh.setDirection(32);
            mehet=true;
          } else if( Field.isIn(elem,field.RIGHT_TILES) ) {
            sh.setDirection(96);
            mehet=true;
          } else {
            sh.setDirection(0); mehet = false;
          }
          rad += 1;

      } while (field.containsImpassableArea(sh.getX()+sh.getWidth()/2,
                                            sh.getY()+sh.getHeight()/2,
                                            sh.getWidth(),
                                            sh.getHeight()) ||
               overlapsSheepdog(sh) ||
               overlapsSheep(sh, i)  || !mehet );

    }
}
